
Summary: Change the default httpd.conf to disable autoindex
Name: zz_apache_no_browsable_directory
Version: 1.0
Release: 3%{?dist}
License: GPL
Group: Fermi
Packager: Pat Riehecky
Vendor: Fermilab
BuildArch: noarch
Requires: augeas coreutils

%description
This modifies the /etc/httpd/conf/httpd.conf file using augeas so that
mod_autoindex does not list your directories out by default.

This will remove 'Indexes' from your 'Options' list for '/var/www/html'
and '/var/www/icons'.  It will also remove 'Indexes' from the sample
options list for mod_userdir (~username directories).

For this change to take effect you must restart httpd.

You can still enable this option with a .htaccess file or by editing the config
file yourself.

To return indexes to working state you will need to add:

 Options +Indexes

to either your apache config or your .htaccess file

See: http://httpd.apache.org/docs/2.2/mod/mod_autoindex.html

%prep
rm -rf $RPM_BUILD_ROOT

%files

%triggerin -- httpd
# not run on upgrades in case you've made changes to things
OK_TO_RUN=0
COMPLEX_CHECK=0

# If we've only got one installed apache, then we are not upgrading
# therefore it should be safe to run this trigger.....
if [[ $2 -eq 1 ]]; then  # httpd count
    OK_TO_RUN=1
fi

# Except, if we've got more than one zz_apache_no_browsable_directory then
# we upgraded zz_apache_no_browsable_directory and this trigger is checked
# because of the new version of this package.
#
# If we've got only one version of zz_apache_no_browsable_directory it is
# safe to run this trigger.
if [[ $1 -eq 1 ]]; then   # zz_apache_no_browsable_directory count
    OK_TO_RUN=1
elif [[ $1 -ne 1 ]]; then # update in progress
    OK_TO_RUN=0

    # if we've only got one instance of httpd, then things might get
    # interesting in a moment.
    if [[ $2 -eq 1 ]]; then   # httpd count
        COMPLEX_CHECK=1
    fi
fi

# It should be impossible to get two versions of this package
# installed without one of them having had the chance to run this trigger
# unless zz_apache_no_browsable_directory is updated at the same time as apache
# is installed for the first time.  The only scenerio where this can actually
# occur is if 'yum update' updates zz_apache_no_browsable_directory and another
# package being updated pulls in httpd as a dependency.
# This isn't an expected case but worth pointing out.
# And worth trying to solve.
#
# How do we check for this?  As near as I can tell we can't.  So we go for
# plan B.  If we are upgrading zz_apache_no_browsable_directory and
# /var/log/httpd/ has no files in it, httpd is chkconfig off, and httpd
# is not running we can guess with some level of faith that it isn't in use
# so making the changes shouldn't hurt anyone since it isn't running
# or configured to run.
#
if [[ $COMPLEX_CHECK -eq 1 ]]; then
    # if apache has never been started /var/log/httpd contains no files
    LOGFILE_COUNT=$(cd /var/log/httpd ; ls |wc -l)
    if [[ "$LOGFILE_COUNT" == "0" ]]; then
        # if we find no commands, just assume we should skip this
        CMD='/bin/true'
        if [[ -x /bin/systemctl ]]; then
            # for systemd systems, this returns 0 if is enabled
            CMD='/bin/systemctl is-enabled httpd.service'
        elif [[ -x /sbin/chkconfig ]]; then
            # for chkconfig systems, this returns 0 if is chkconfig on
            CMD='/sbin/chkconfig --list httpd |grep :on'
        fi
        OUTPUT=$($CMD >/dev/null 2>&1)
        if [[ $? -ne 0 ]]; then
            RUNNING_HTTPD=$(ps -ef |grep httpd|grep -v grep| wc -l)
            if [[ "$RUNNING_HTTPD" == "0" ]]; then
                # if we got here
                #     there were no logs
                #     the service isn't enabled
                #     there are no 'httpd' processes running
                # if it isn't safe to edit this config file now, I'm not sure
                # when it would be
                OK_TO_RUN=1
            fi
        fi
    fi
fi


# if OK_TO_RUN == 1 then we've never run this trigger before
# or at the very least it shouldn't be harmful to run it now
if [[ $OK_TO_RUN -eq 1 ]]; then

    cp -pa /etc/httpd/conf/httpd.conf /etc/httpd/conf/httpd.conf.pre_zz

    TIMESTAMP=`date`
    STAMP="On $TIMESTAMP %{name}"

    TMPFILE=`mktemp`

    # To understand what the lines below are doing you will need to review the
    # augeas documentation.
    # The short version is:
    #
    #   *[arg = 'x']  will match an item that has a sub item named 'arg' with a
    #                 value of x.  It will match as many as there are.
    #
    #   arg[. = 'Indexes']  will match the element of an array whose value is
    #                       Indexes

    # the trick with the comments is far more complex
    #
    #  Insert the comment before the node that begins the line
    #  Find the comment block before the node
    #  Find the end element
    #  Then set it

    # expect the quotation marks to be either missing or "
    cat > $TMPFILE <<CHANGEWEBROOT
ins #comment before /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/html["]*')]/*[arg = 'Indexes']
set /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/html["]*')]/#comment[following-sibling::*[1][arg = 'Indexes']] "$STAMP removed Indexes from Options"
rm /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/html["]*')]/*[arg = 'Indexes']/arg[. = 'Indexes']

ins #comment before /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/html["]*')]/*[. = 'AllowOverride']
set /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/html["]*')]/#comment[following-sibling::*[1][arg = 'None']] "$STAMP added Indexes to AllowOverride"

set /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/html["]*')]/*[. = 'AllowOverride']/*[. = 'None'] Indexes

save

CHANGEWEBROOT

    # remove the 'Indexes' option from /var/www/html Options list
    # Allow .htaccess to set 'Indexes'
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE

    ########################################################################

    TMPFILE=`mktemp`
    # expect the quotation marks to be either missing or "
     cat > $TMPFILE <<CHANGEICONS
ins #comment before /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/icons["]*')]/*[arg = 'Indexes']
set /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/icons["]*')]/#comment[following-sibling::*[1][arg = 'Indexes']] "$STAMP removed Indexes from Options"

rm /files/etc/httpd/conf/httpd.conf/*[arg =~ regexp('["]*/var/www/icons["]*')]/*[arg = 'Indexes']/arg[. = 'Indexes']

save

CHANGEICONS

    # remove the 'Indexes' option from /var/www/icons Options list
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE

    ########################################################################
    TMPFILE=`mktemp`
    # If we happen to find the default options for mod_userdir
    # remove Indexes from the list, but add it to AllowOverride
     cat > $TMPFILE <<CHANGEUSERDIR
set /files/etc/httpd/conf/httpd.conf/*[ . = 'AllowOverride FileInfo AuthConfig Limit' ] 'AllowOverride FileInfo AuthConfig Limit Indexes'
set /files/etc/httpd/conf/httpd.conf/*[ . = 'Options MultiViews Indexes SymLinksIfOwnerMatch IncludesNoExec' ] 'Options MultiViews SymLinksIfOwnerMatch IncludesNoExec'

save

CHANGEUSERDIR

    # remove the 'Indexes' option from mod_userdir sample Options list
    augtool < $TMPFILE >/dev/null 2>&1
    rm -f $TMPFILE

fi



##########################################################################
%changelog
* Tue Dec 4 2012 Pat Riehecky <riehecky@fnal.gov> 1.0-3
- Sets up mod_userdir with no indexes too
- Now leaves a comment behind saying what it did
- Now archives httpd.conf before making the edits

* Fri Nov 30 2012 Pat Riehecky <riehecky@fnal.gov> 1.0-2
- Improved internal documentation

* Wed Nov 28 2012 Pat Riehecky <riehecky@fnal.gov> 1.0-1
- Trigger is now far more selective about when it runs

* Tue Nov 27 2012 Pat Riehecky <riehecky@fnal.gov> 1.0-0
- Initial build at the request of computing security
